package samples.circle 
{
	
	import biga.CrossHair;
	import biga.shapes2d.Circle;
	import biga.utils.CircleUtils;
	import biga.utils.GeomUtils;
	import com.bit101.components.CheckBox;
	import com.bit101.components.HUISlider;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class CircleExtraSample extends Sprite 
	{
		private var c0:CrossHair;
		private var c1:CrossHair;
		private var circle0:Circle;
		private var circle1:Circle;
		
		private var circleRadius:HUISlider;
		private var homotheticCenters:CheckBox;
		private var radicalLines:CheckBox;
		private var inversionPoint:CheckBox;
		
		public function CircleExtraSample() 
		{
			c0 = new CrossHair( 0, 12 );
			c0.x = 250;
			c0.y = 250;
			addChild( c0 );
			
			c1 = new CrossHair( 0xFF0000, 12 );
			c1.x = 400;
			c1.y = 400;
			addChild( c1 );
			
			circle0 = new Circle( c0.x, c0.y, 100 );
			circle1 = new Circle( c1.x, c1.y, 50 );
			
			var ox:int = 10;
			var oy:int = -10;
			
			circleRadius = new HUISlider( this, ox, oy += 20, 'radius' );
			circleRadius.maximum = 200;
			circleRadius.value = circle0.radius;
			
			oy += 20;
			homotheticCenters = new CheckBox( this, ox, oy += 20, 'homothetic centers' );
			radicalLines = new CheckBox( this, ox, oy += 20, 'radical line' );
			inversionPoint = new CheckBox( this, ox, oy += 20, 'polar and inversion point of the mouse' );
			
			
			addEventListener( Event.ENTER_FRAME, update );
			
		}
		
		private function update( e:Event ) :void
		{
			circle0.x = c0.x;
			circle0.y = c0.y;
			circle1.x = c1.x;
			circle1.y = c1.y;
			circle0.radius = circleRadius.value;
			
			var v:Vector.<Point>;
			var p:Point;
			var mouse:Point = new Point( mouseX, mouseY );
			
			graphics.clear();
			
			graphics.lineStyle( 0 );
			circle0.draw( graphics );
			circle1.draw( graphics );
			graphics.endFill();
			
			if ( homotheticCenters.selected )
			{
				
				graphics.lineStyle( 0, 0, 0 );
				graphics.beginFill( 0x66CC33 );
				v = CircleUtils.homotheticCenters( circle0, circle1 );
				
				if ( v != null )
				{
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 5 );
					graphics.endFill();
					
					graphics.lineStyle( 2, 0x66CC33 );
					graphics.moveTo( circle0.x, circle0.y );
					graphics.lineTo( v[1].x, v[1].y );
					
					p = CircleUtils.closestPointOnCircle( mouse, circle0 );
					graphics.drawCircle( p.x, p.y, 5 );
					graphics.moveTo( p.x, p.y );
					graphics.lineTo( v[1].x, v[1].y );
					graphics.moveTo( p.x, p.y );
					graphics.lineTo( v[0].x, v[0].y );
					
					p = CircleUtils.setPointAtAngle( GeomUtils.angle( circle0, p ), circle1 );
					graphics.lineTo( p.x, p.y );
					graphics.drawCircle( p.x, p.y, 5 );
					
				}
			}
			
			if ( radicalLines.selected )
			{
				v = CircleUtils.radicalLine( circle0, circle1 );
				if ( v != null )
				{
					graphics.lineStyle( 2, 0x6633AA );
					graphics.moveTo( v[ 0 ].x, v[ 0 ].y );
					graphics.lineTo( v[ 1 ].x, v[ 1 ].y );
					graphics.lineStyle( 0,0,0 );
					
					graphics.beginFill( 0x6633AA );
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 4 );
					graphics.endFill();
				}
			}
			
			if ( inversionPoint.selected )
			{
				v = CircleUtils.poleFromPoint( mouse, circle0, 50 );
				if ( v != null )
				{
					graphics.lineStyle( 2, 0x6633AA );
					graphics.moveTo( v[ 0 ].x, v[ 0 ].y );
					graphics.lineTo( v[ 1 ].x, v[ 1 ].y );
					graphics.lineStyle( 0, 0, 0 );
				}
				
				p = CircleUtils.inversionPoint( mouse, circle0 );
				if ( p != null )
				{
					graphics.beginFill( 0x6633AA );
					graphics.drawCircle( p.x, p.y, 4 );
					graphics.endFill();
				}
			}
			
		}
		
	}

}