package samples.circle 
{
	
	import biga.CrossHair;
	import biga.shapes2d.Circle;
	import biga.shapes2d.Edge;
	import biga.utils.CircleUtils;
	import com.bit101.components.CheckBox;
	import com.bit101.components.HUISlider;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class CircleIntersectionSample extends Sprite 
	{
		private var c0:CrossHair;
		private var c1:CrossHair;
		private var circle0:Circle;
		private var circle1:Circle;
		private var edge:Edge;
		private var ec1:CrossHair;
		private var ec0:CrossHair;
		
		private var circleRadius:HUISlider;
		
		private var circlesIntersection:CheckBox;
		private var lineCircleIntersection:CheckBox;
		private var segmentCircleIntersection:CheckBox;
		
		
		public function CircleIntersectionSample() 
		{
			c0 = new CrossHair( 0, 12 );
			c0.x = 250;
			c0.y = 250;
			addChild( c0 );
			
			c1 = new CrossHair( 0xFF0000, 12 );
			c1.x = 400;
			c1.y = 400;
			addChild( c1 );
			
			circle0 = new Circle( c0.x, c0.y, 100 );
			circle1 = new Circle( c1.x, c1.y, 50 );
			
			ec0 = CrossHair( addChild( new CrossHair( 0xBBBBBB )));
			ec0.point = new Point( 250, 50 );
			ec1 = CrossHair( addChild( new CrossHair( 0xBBBBBB )));
			ec1.point = new Point(250, 450 );
			edge = new Edge( ec0.point , ec1.point );
			
			
			var ox:int = 10;
			var oy:int = -10;
			
			circleRadius = new HUISlider( this, ox, oy += 20, 'radius' );
			circleRadius.maximum = 200;
			circleRadius.value = circle0.radius;
			
			oy += 20;
			
			circlesIntersection = new CheckBox( this, ox, oy += 20, 'circles intersection' );
			lineCircleIntersection = new CheckBox( this, ox, oy += 20, 'line-circle intersection' );
			segmentCircleIntersection = new CheckBox( this, ox, oy += 20, 'segment-circle intersection' );
			
			addEventListener( Event.ENTER_FRAME, update );
			
		}
		
		private function update( e:Event ) :void
		{
			circle0.x = c0.x;
			circle0.y = c0.y;
			circle1.x = c1.x;
			circle1.y = c1.y;
			
			edge.p0 = ec0.point;
			edge.p1 = ec1.point;
			
			circle0.radius = circleRadius.value;
			
			var v:Vector.<Point>;
			var p:Point;
			var mouse:Point = new Point( mouseX, mouseY );
			
			graphics.clear();
			
			graphics.lineStyle( 0 );
			if ( ( CircleUtils.containsCircle( circle0, circle1 ) ) )
			{
				graphics.lineStyle( 0, 0, 0 );
				graphics.beginFill( 0xFF1E49 );
			}
			circle0.draw( graphics );
			circle1.draw( graphics );
			graphics.endFill();
			
			if ( circlesIntersection.selected ) 
			{
				graphics.lineStyle( 2, 0xFF1E49 );
				v = CircleUtils.circlesIntersection( circle0, circle1  );
				if ( v != null )
				{
					
					graphics.moveTo( v[0].x, v[0].y );
					graphics.lineTo( v[1].x, v[1].y );
					
				}
			}
			
			
			if ( lineCircleIntersection.selected || segmentCircleIntersection.selected )
			{
				ec0.visible = ec1.visible = true;
				graphics.lineStyle( 1, 0xDDDDDD );
				edge.draw( graphics );
				
				graphics.lineStyle( 0,0,0 );
				if ( lineCircleIntersection.selected )
				{
					graphics.beginFill( 0xFF0000 );
					v = CircleUtils.lineCircleIntersection( edge.p0, edge.p1, circle0 );
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 4 );
					graphics.endFill();
					
					graphics.beginFill( 0xFF0000 );
					v = CircleUtils.lineCircleIntersection( edge.p0, edge.p1, circle1 );
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 4 );
					graphics.endFill();
				}
				
				
				if ( segmentCircleIntersection.selected )
				{
					graphics.lineStyle( 0, 0xFF0000 );
					v = CircleUtils.segmentCircleIntersection( edge.p0, edge.p1, circle0 );
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 20 );
					v = CircleUtils.segmentCircleIntersection( edge.p0, edge.p1, circle1 );
					for each ( p in v ) graphics.drawCircle( p.x, p.y, 20 );
					
				}
			}
			else
			{
				ec0.visible = ec1.visible = false;
			}
			
			
		}
		
	}

}