package samples.circle 
{
	
	import biga.CrossHair;
	import biga.shapes2d.Arc;
	import biga.shapes2d.Circle;
	import biga.utils.ArcUtils;
	import biga.utils.CircleUtils;
	import com.bit101.components.CheckBox;
	import com.bit101.components.HUISlider;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class TangentSample extends Sprite 
	{
		private var c0:CrossHair;
		private var c1:CrossHair;
		private var circle0:Circle;
		private var circle1:Circle;
		private var arc:Arc;
		
		private var circleRadius:HUISlider;
		private var pointTangents:CheckBox;
		private var circlesTangents:CheckBox;
		private var circlesDistance:CheckBox;
		
		private var pointTangentsArc:CheckBox;
		private var pointTangentsArcInvert:CheckBox;
		private var circlesTangentsArc:CheckBox;
		private var circlesTangentsArcInvert:CheckBox;
		
		public function TangentSample() 
		{
			c0 = new CrossHair( 0, 12 );
			c0.x = 250;
			c0.y = 250;
			addChild( c0 );
			
			c1 = new CrossHair( 0xFF0000, 12 );
			c1.x = 400;
			c1.y = 400;
			addChild( c1 );
			
			circle0 = new Circle( c0.x, c0.y, 100 );
			circle1 = new Circle( c1.x, c1.y, 50 );
			arc = new Arc( circle0.x, circle0.y, circle0.radius );
			
			var ox:int = 10;
			var oy:int = -10;
			
			circleRadius = new HUISlider( this, ox, oy += 20, 'radius' );
			circleRadius.maximum = 200;
			circleRadius.value = circle0.radius;
			
			oy += 20;
			
			circlesDistance = new CheckBox( this, ox, oy += 20, 'circles distance' );
			circlesDistance.selected = true;
			
			pointTangents = new CheckBox( this, ox, oy += 20, 'point tangents' );
			pointTangentsArc = new CheckBox( this, ox + 10, oy += 20, 'draw arc' );
			pointTangentsArcInvert = new CheckBox( this, ox + 110, oy, 'invert' );
			
			circlesTangents = new CheckBox( this, ox, oy += 20, 'circle tangents' );
			circlesTangentsArc = new CheckBox( this, ox + 10, oy += 20, 'draw arc' );
			circlesTangentsArcInvert = new CheckBox( this, ox + 110, oy, 'invert' );
			
			addEventListener( Event.ENTER_FRAME, update );
			
		}
		
		private function update( e:Event ) :void
		{
			circle0.x = arc.x = c0.x;
			circle0.y = arc.y = c0.y;
			circle1.x = c1.x;
			circle1.y = c1.y;
			
			circle0.radius = arc.radius = circleRadius.value;
			
			var v:Vector.<Point>;
			var p:Point;
			var mouse:Point = new Point( mouseX, mouseY );
			
			graphics.clear();
			
			if ( circlesDistance.selected ) 
			{
				graphics.lineStyle( 2, 0x38ABE8 );
				if ( CircleUtils.circlesDistance( circle0, circle1 ) < 0 )
				{
					graphics.lineStyle( 2, 0xFF1E49 );
				}
				v = CircleUtils.circlesDistanceSegment( circle0, circle1 );
				graphics.moveTo( v[0].x, v[0].y );
				graphics.lineTo( v[1].x, v[1].y );
			}
			
			if ( pointTangents.selected ) 
			{
				graphics.lineStyle( 2, 0x85FF32 );
				v = CircleUtils.tangentsToPoint( new Point( c1.x, c1.y ), circle0 );
				if ( v != null && v.length > 1 )
				{
					
					graphics.moveTo( c1.x, c1.y );
					graphics.lineTo( v[0].x, v[0].y );
					graphics.moveTo( c1.x, c1.y );
					graphics.lineTo( v[1].x, v[1].y );
					
					if ( pointTangentsArc.selected ) 
					{
						graphics.lineStyle( 4, 0x85FF32 );
						ArcUtils.init( arc, v[ 0 ], v[1], circle0 );
						if ( pointTangentsArcInvert.selected ) ArcUtils.invert( arc );
						arc.draw( graphics );
					}
				}
			}
			
			
			if ( circlesTangents.selected ) 
			{
				graphics.lineStyle( 4, 0xFFBF0E );
				v = CircleUtils.tangentsToCircle( circle0, circle1  );
				if ( v != null )
				{
					graphics.moveTo( v[0].x, v[0].y );
					graphics.lineTo( v[1].x, v[1].y );
					
					graphics.moveTo( v[2].x, v[2].y );
					graphics.lineTo( v[3].x, v[3].y );
					
					if ( circlesTangentsArc.selected )
					{
						
						var biggerCircle:Circle = circle0.radius >= circle1.radius ? circle0 : circle1;
						var smallerCircle:Circle = circle0.radius < circle1.radius ? circle0 : circle1;
						
						ArcUtils.init( arc, v[ 0 ], v[ 2 ], biggerCircle );
						ArcUtils.invert( arc );
						if ( circlesTangentsArcInvert.selected )ArcUtils.invert( arc );
						arc.draw( graphics );
						
						ArcUtils.init( arc, v[ 1 ], v[ 3 ], smallerCircle );
						if ( circlesTangentsArcInvert.selected ) ArcUtils.invert( arc );
						arc.draw( graphics );
						
					}
				}
			}
			
			graphics.lineStyle( 0 );
			if ( ( CircleUtils.containsCircle( circle0, circle1 ) ) )
			{
				graphics.lineStyle( 0, 0, 0 );
				graphics.beginFill( 0xFF1E49 );
			}
			circle0.draw( graphics );
			circle1.draw( graphics );
			graphics.endFill();
			
			
		}
		
	}

}