package samples.lines 
{
	import biga.CrossHair;
	import biga.shapes2d.Circle;
	import biga.shapes2d.Edge;
	import biga.shapes2d.Triangle;
	import biga.utils.GeomUtils;
	import biga.utils.TriangleUtils;
	import com.bit101.components.CheckBox;
	import com.bit101.components.Label;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.geom.Point;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class TriangleSample extends Sprite 
	{
		
		private var triangle:Triangle;
		
		private var h0:CrossHair;
		private var h1:CrossHair;
		private var h2:CrossHair;
		
		private var medians:CheckBox;
		private var centroid:CheckBox;
		private var altitudes:CheckBox;
		private var orthocenter:CheckBox;
		private var perpBisector:CheckBox;
		private var circumcircle:CheckBox;
		private var bisector:CheckBox;
		private var incircle:CheckBox;
		private var eulerLine:CheckBox;
		private var eulerCircle:CheckBox;
		private var excircle:CheckBox;
		private var area:Label;
		private var inside:Label;
		private var culling:Label;
		
		
		public function TriangleSample() 
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			
			h0 = new CrossHair( 0xFF0000 );
			h0.x = 250;
			h0.y = 100; 
			h1 = new CrossHair( 0x00FF00 );
			h1.x = 400;
			h1.y = 350; 
			h2 = new CrossHair( 0x0000FF );
			h2.x = 100;
			h2.y = 350; 
			
			triangle = new Triangle( h0.point, h1.point, h2.point );
			
			var ox:int = 10;
			var oy:int = -10;
			
			medians 			= new CheckBox( this, ox, oy += 20, 'medians' );
			centroid 			= new CheckBox( this, ox + 10, oy += 20, 'centroid' );
			altitudes 			= new CheckBox( this, ox, oy += 20, 'altitudes' );
			orthocenter			= new CheckBox( this, ox + 10, oy += 20, 'orthocenter' );
			bisector 			= new CheckBox( this, ox, oy += 20, 'bisector' );
			incircle 			= new CheckBox( this, ox + 10, oy += 20, 'incircle' );
			excircle 			= new CheckBox( this, ox + 10, oy += 20, 'excircles' );
			perpBisector 		= new CheckBox( this, ox, oy += 20, 'perpendicular bisector' );
			circumcircle 		= new CheckBox( this, ox + 10, oy += 20, 'circumcircle' );
			eulerLine 			= new CheckBox( this, ox, oy += 20, 'euler line' );
			eulerCircle 		= new CheckBox( this, ox + 10, oy += 20, 'euler circle' );
			
			area				= new Label( this, ox, oy += 20, '' );
			inside				= new Label( this, ox, oy += 20, '' );
			culling				= new Label( this, ox, oy += 20, '' );
			
			addChild( h0 );
			addChild( h1 );
			addChild( h2 );
			
			addEventListener( Event.ENTER_FRAME, oef );
			
		}
		
		private function oef(event:Event):void 
		{
			
			graphics.clear();
			triangle.p0 =  h0.point;
			triangle.p1 =  h1.point;
			triangle.p2 =  h2.point;
			
			area.text = 'area: ' + .5 * GeomUtils.toFixed( triangle.edges[0].length + triangle.edges[1].length + triangle.edges[2].length, 10 );
			
			inside.text = 'the mouse is outside';
			if( triangle.contains( new Point( stage.mouseX, stage.mouseY ) ) )inside.text = 'the mouse is inside';
			
			
			culling.text = 'culling: front'
			if ( triangle.culling ) culling.text = 'culling: back';
			
			var ip:Point;
			var p:Point;
			var c:Circle;
			
			var e0:Edge;
			var e1:Edge;
			var e2:Edge;
			
			if ( centroid.selected )
			{
				
				graphics.beginFill( 0x00CC00 );
				ip = TriangleUtils.centroid( triangle );
				graphics.drawCircle( ip.x, ip.y, 5 );
				graphics.endFill();
				
			}
			
			
			if ( medians.selected )
			{
				graphics.lineStyle( 2, 0x00CC00 );
				
				e0 = TriangleUtils.median( triangle, 0 );
				e0.draw( graphics );
				e1 = TriangleUtils.median( triangle, 1 );
				e1.draw( graphics );
				e2 = TriangleUtils.median( triangle, 2 );
				e2.draw( graphics );
				
			}
			
			if ( altitudes.selected )
			{
				graphics.lineStyle( 2, 0xFFCC00 );
				
				e0 = TriangleUtils.altitude( triangle, 0 );
				e0.draw( graphics );
				
				e1 = TriangleUtils.altitude( triangle, 1 );
				e1.draw( graphics );
				
				e2 = TriangleUtils.altitude( triangle, 2 );
				e2.draw( graphics );
				
				ip = TriangleUtils.orthocenter( triangle );
				
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( e0.p1.x, e0.p1.y );
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( e1.p1.x, e1.p1.y );
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( e2.p1.x, e2.p1.y );
				
			}
			
			if ( orthocenter.selected )
			{
				
				ip = TriangleUtils.orthocenter( triangle );
				graphics.beginFill( 0xFFCC00 );
				graphics.drawCircle( ip.x, ip.y, 5 );
				graphics.endFill();
				
			}
			
			
			if ( bisector.selected )
			{
				graphics.lineStyle( 2, 0x008080 );
				
				e0 = TriangleUtils.bissector( triangle, 0 );
				e0.draw( graphics );
				
				e1 = TriangleUtils.bissector( triangle, 1 );
				e1.draw( graphics );
				
				e2 = TriangleUtils.bissector( triangle, 2 );
				e2.draw( graphics );
				
			}
			
			if ( incircle.selected )
			{
				
				graphics.lineStyle( 2, 0x008080 );
				c = TriangleUtils.incircle( triangle );
				if ( c != null )
				{
					c.draw( graphics );
					graphics.beginFill( 0x008080 );
					graphics.drawCircle( c.x, c.y, 5 );
					graphics.endFill();
				}
				
			}
			
			if ( excircle.selected )
			{
				graphics.lineStyle( 2, 0x008080 );
				
				c = TriangleUtils.excircle( triangle, 0 );
				graphics.drawCircle( c.x, c.y, c.radius );
				graphics.moveTo( c.x, c.y );
				p = c;
				
				c = TriangleUtils.excircle( triangle, 1 );
				graphics.lineTo( c.x, c.y );
				graphics.drawCircle( c.x, c.y, c.radius );
				graphics.moveTo( c.x, c.y );
				
				c = TriangleUtils.excircle( triangle, 2 );
				graphics.lineTo( c.x, c.y );
				graphics.drawCircle( c.x, c.y, c.radius );
				graphics.moveTo( c.x, c.y );
				
				graphics.lineTo( p.x, p.y );
				
				
			}
			
			if ( perpBisector.selected )
			{
				
				graphics.lineStyle( 2, 0x330066 );
				c = TriangleUtils.circumcircle( triangle );
				ip = new Point( c.x, c.y );
				p = triangle.edges[0].center;
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( p.x, p.y );
				
				p = triangle.edges[1].center;
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( p.x, p.y );
				
				p = triangle.edges[2].center;
				graphics.moveTo( ip.x, ip.y );
				graphics.lineTo( p.x, p.y );
			}
			
			if ( circumcircle.selected )
			{
				graphics.lineStyle( 2, 0x330066 );
				c = TriangleUtils.circumcircle( triangle );
				graphics.drawCircle( c.x, c.y, c.radius );
				graphics.beginFill( 0x330066 );
				graphics.drawCircle( c.x, c.y, 5 );
				graphics.endFill();
			}
			
			if ( eulerLine.selected )
			{
				
				graphics.lineStyle( 3, 0xFF0000 );
				e0 = TriangleUtils.eulerLine( triangle );
				e0.draw( graphics );
				
			}
			if ( eulerCircle.selected )
			{
				
				graphics.lineStyle( 2, 0xFF0000 );
				c = TriangleUtils.eulerCircle( triangle );
				c.draw( graphics );
				graphics.beginFill( 0xFF0000 );
				graphics.drawCircle( c.x, c.y, 5 );
				graphics.endFill();
				
			}
			
			
			
			graphics.lineStyle( 0 );
			if ( triangle.culling ) graphics.beginFill( 0, .05 );			
			triangle.draw( graphics );
			graphics.endFill();
			
		}
		
	}

}